#include <p18f452.h>
#include <delays.h>

#include "LCD.H"
#include "Curseurs.h"
#include "font_5x7.h"

void LCD_Write_Command(unsigned char Data);
void LCD_Send_Data(unsigned char Data);
void LCD_Init(void);
void LCD_Clear(void);
void LCD_GotoXY(unsigned char X,unsigned char Y);
void LCD_Putchar(char ch);
unsigned char LCD_ReadData(void);
void LCD_PutPixel(unsigned char X, unsigned char Y);
void LCD_PutPasPixel(unsigned char X, unsigned char Y);


void LCD_Init (void)
{
	//Configuration de toutes les broches de l'afficheur LCD en sortie
	TRIS_LCD_CSA		= 0;	//En sortie, pour le controlleur A
	TRIS_LCD_CSB		= 0;	//En sortie, pour le controlleur B
	TRIS_LCD_RS			= 0;	//En sortie, pour choisir le mode de fonctionnement
	TRIS_LCD_RW			= 0;	//En sortie, pour choisir le mode de fonctionnement
	TRIS_LCD_ENABLE		= 0;	//En sortie, pour commander le ENABLE
	TRIS_LCD_DATA		= 0;	//En sortie, pour envoyer une donne
	TRIS_LCD_RESET		= 0;	//En sortie, pour commander le reset
	TRIS_LCD_BACKLIGHT	= 0;	//En sortie, pour commander le rtroclairage

	LCD_CSA				= 1;	//Slectionne le controlleur A
	LCD_CSB				= 1;	//Slectionne le controlleur B
	LCD_ENABLE			= 0;	//Dsactive la broche ENABLE
	LCD_RESET			= 0;	//Mettre le RESET  0
	Delay100TCYx(1);			//Attendre 100 cycles
	LCD_RESET			= 1;	//Mettre le RESET  1
	LCD_Clear();				//Vide l'cran
	LCD_Write_Command(0x3F);	//Allume l'cran
}

//fonction permettant l'envoi d'une commande  l'cran LCD
void LCD_Write_Command(unsigned char Data)
{
	//Afficheur en mode Commande
	LCD_RS				= 0;	
	LCD_RW				= 0;
	//

	LCD_DATA			= Data;	//
	Delay100TCYx(2);			//Attendre 200cycles
	LCD_ENABLE			= 1;	//Active la broche ENABLE
	Delay100TCYx(2);			//Attendre 200cycles
	LCD_ENABLE			= 0;	//Dsactive la broche ENABLE	
}

//fonction permettant l'affichage de donne sur l'cran LCD
void LCD_Send_Data(unsigned char Data)
{
	//Afficheur en mode criture
	LCD_RS				= 1;	
	LCD_RW				= 0;
	//

	LCD_DATA			= Data;	//Mise en place de la donne sur le port de donne
	Delay100TCYx(2);			//Attendre 200cycles
	LCD_ENABLE			= 1;	//Active la broche ENABLE
	Delay100TCYx(2);			//Attendre 200cycles
	LCD_ENABLE			= 0;	//Dsactive la broche ENABLE	
}

void LCD_Clear(void)
{
	unsigned char a,b;
	LCD_CSA				= 1;		//Slectionne le controlleur A
	LCD_CSB				= 1;		//Slectionne le controlleur B
	for(a=0;a<8;a++)				//Effacement des 8 pages des 2 parties
	{
		LCD_Write_Command(0xB8|a);	//Permet d'aller a toutes les pages diffrentes
		for(b=0;b<64;b++)			//Efface la page courante
			LCD_Send_Data(0x00);//	
	}
	LCD_Write_Command(0xB8);		//Retourne a la page 0
}

void LCD_GotoXY(unsigned char X,unsigned char Y)
{
	if(X<64)							
	{
		//Slectionne la page B (cot gauche de l'cran)
		LCD_CSA		= 0;				
		LCD_CSB		= 1;
		//
						
		LCD_Write_Command(0xB8|Y);		//Slectionne la page		
		LCD_Write_Command(0x40|X);		//Slectionne la ligne
	}
	else
	{
		//Slectionne la page A (cot droit de l'cran)
		LCD_CSA		= 1;
		LCD_CSB		= 0;
		//
	
		LCD_Write_Command(0xB8|Y);		//Slectionne la page
		LCD_Write_Command(0x40|(X-64));	//Slectionne la ligne
	}
}


void LCD_Putchar(char ch)
{
	unsigned char o;
	int index;						//Variable permettant de couvrir le tableau des caracteres
	index=(ch-32);					//Dcalage de -32 car le premier caractere du tableau de caractere est espace (ASCII : 32)
	index=index*5;					//Permet de ne slectionne que 1 case sur 5 du tableau, chaque caractere tant sur 5 cases
	for(o=0;o<5;o++)				//Affiche le caractere
	{
		LCD_Send_Data(Font[index]);	//envoie une case du tableau pour crire une colonne
		index++;					//Choisi la case suivante
	}
	LCD_Send_Data(0x00);			//insere un blanc entre chaque caractere
}	

unsigned char LCD_ReadData(void)
{
	unsigned char DataPort;
	TRIS_LCD_DATA		= 0xFF;		//Configuration de tout les ports de donne en entre
	//Afficheur en mode Lecture
	LCD_RS				= 1;		
	LCD_RW				= 1;		
	//

	LCD_ENABLE			= 1;		//Active la broche ENABLE
	Delay10TCYx(5);					//Attendre 50 cycles
	LCD_ENABLE			= 0;		//Desactive la broche ENABLE
	DataPort			= LCD_DATA;	//Envoie de la donne prsente sur la colonne courante au C
	TRIS_LCD_DATA		= 0;		//Config des ports de donne en entre
	return(DataPort);
}
	
//Allume un pixel sur l'cran  la position X,Y

void LCD_PutPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val, page=Y>>3;	//Calcule la page courante
	LCD_GotoXY(X,page);				//Place le curseur sur l'octet courant
	Val=LCD_ReadData();				//Lit l'octet courant
	Val=LCD_ReadData();				//?????? Bug ???????
	//Ralise le masque
	Val=Val|(0x01<<(Y%8));
	//Val=0x01;
	LCD_GotoXY(X,page);				//Replace le curseur
	LCD_Send_Data(Val);				//Envoie la donne transforme
}

void LCD_PutPasPixel(unsigned char X, unsigned char Y)
{
	unsigned char Val, page=Y>>3;	//Calcule la page courante
	LCD_GotoXY(X,page);				//Place le curseur sur l'octet courant
	Val=LCD_ReadData();				//Lit l'octet courant
	Val=LCD_ReadData();				//?????? Bug ???????
	Val=Val&(~(0x01<<(Y%8)));
	LCD_GotoXY(X,page);				//Replace le curseur
	LCD_Send_Data(Val);				//Envoie la donne transforme
}	


